from datetime import datetime
from pathlib import Path

import pygfried

conftest_path = Path(__file__).parent / "conftest.py"


def test_version(siegfried_version):
    assert pygfried.version() == siegfried_version


def test_identify():
    result = pygfried.identify(str(conftest_path))
    assert result == "fmt/938"


def test_identify_detailed(siegfried_version):
    result = pygfried.identify(str(conftest_path), detailed=True)

    assert result["siegfried"] == siegfried_version
    parse_iso_date(result["scandate"])
    assert result["signature"] == "default.sig"
    parse_iso_date(result["created"])
    assert len(result["identifiers"]) > 0
    assert len(result["files"]) == 1

    finfo = result["files"][0]
    parse_iso_date(str(finfo.pop("modified")))
    assert finfo == {
        "filename": str(conftest_path),
        "filesize": conftest_path.stat().st_size,
        "errors": "",
        "matches": [
            {
                "ns": "pronom",
                "id": "fmt/938",
                "format": "Python Source Code File",
                "version": "",
                "mime": "",
                "class": "Text (Structured)",
                "basis": "extension match py",
                "warning": "match on extension only",
            }
        ],
    }


def parse_iso_date(date_string):
    """Parse ISO date string with compatibility for Python 3.9-3.11"""
    try:
        return datetime.fromisoformat(date_string)
    except ValueError:
        if date_string.endswith("Z"):
            date_string = date_string[:-1] + "+00:00"
        try:
            return datetime.fromisoformat(date_string)
        except ValueError:
            if "." in date_string:
                parts = date_string.split(".")
                if len(parts) == 2:
                    microsecs = parts[1]
                    if "+" in microsecs:
                        microsecs, tz = microsecs.split("+", 1)
                        microsecs = microsecs[:6].ljust(6, "0")
                        date_string = f"{parts[0]}.{microsecs}+{tz}"
                    elif microsecs.endswith("+00:00"):
                        microsecs = microsecs[:-6]
                        microsecs = microsecs[:6].ljust(6, "0")
                        date_string = f"{parts[0]}.{microsecs}+00:00"
                    else:
                        microsecs = microsecs[:6].ljust(6, "0")
                        date_string = f"{parts[0]}.{microsecs}"
            return datetime.fromisoformat(date_string)



import json
import subprocess
from pathlib import Path

import pytest


@pytest.fixture
def siegfried_version():
    modname = "github.com/richardlehane/siegfried"
    root_dir = Path(__file__).parent
    result = subprocess.run(
        ["go", "list", "-m", "-json", modname],
        cwd=root_dir,
        capture_output=True,
        text=True,
        check=True,
    )
    module_info = json.loads(result.stdout)
    if module_info.get("Path") == modname:
        return module_info.get("Version").lstrip("v")  # Remove the "v" prefix.
    raise ValueError("Could not find siegfried in the go.mod file.")



from typing import Literal
from typing import TypedDict
from typing import overload

class GoError(Exception):
    """Exception raised when Go code encounters an error."""

SimpleIdentifyResult = Literal["UNKNOWN"] | str | None

class Match(TypedDict):
    ns: str
    id: str
    format: str
    version: str
    mime: str
    basis: str
    warning: str

Match.__annotations__["class"] = str

class File(TypedDict):
    filename: str
    filesize: int
    modified: str
    errors: str
    matches: list[Match]

class Identifier(TypedDict):
    name: str
    details: str

class DetailedIdentifyResult(TypedDict):
    siegfried: str
    scandate: str
    signature: str
    created: str
    identifiers: list[Identifier]
    files: list[File]

@overload
def identify(path: str, detailed: Literal[True]) -> DetailedIdentifyResult: ...
@overload
def identify(path: str, detailed: Literal[False] = False) -> SimpleIdentifyResult: ...
def version() -> str: ...



import platform
import sys

from setuptools import Extension
from setuptools import setup

if sys.platform != "win32" and platform.python_implementation() == "CPython":
    try:
        from setuptools.command.bdist_wheel import bdist_wheel as _bdist_wheel
    except ImportError:
        cmdclass = {}
    else:

        class bdist_wheel(_bdist_wheel):
            def finalize_options(self) -> None:
                self.py_limited_api = f"cp3{sys.version_info[1]}"
                super().finalize_options()

        cmdclass = {"bdist_wheel": bdist_wheel}
else:
    cmdclass = {}

setup(
    ext_modules=[
        Extension(
            "pygfried",
            ["pylib/main.go"],
            py_limited_api=True,
            define_macros=[("Py_LIMITED_API", None)],
        ),
    ],
    cmdclass=cmdclass,
    build_golang={"root": "github.com/artefactual-labs/pygfried"},
    packages=["pygfried"],
    package_data={"pygfried": ["py.typed", "__init__.pyi"]},
    use_scm_version=True,
)
